Package: tximeta
Version: 1.16.0
Title: Transcript Quantification Import with Automatic Metadata
Description: Transcript quantification import from Salmon and
    alevin with automatic attachment of transcript ranges and
    release information, and other associated metadata. De novo
    transcriptomes can be linked to the appropriate sources with
    linkedTxomes and shared for computational reproducibility.
Authors@R: c(
    person("Michael", "Love", email="michaelisaiahlove@gmail.com", role = c("aut","cre")),
    person("Charlotte", "Soneson", role = c("aut","ctb")),
    person("Peter", "Hickey", role = c("aut","ctb")),
    person("Rob", "Patro", role = c("aut","ctb")),
    person("NIH NHGRI", role="fnd"),
    person("CZI", role="fnd")
    )
Maintainer: Michael Love <michaelisaiahlove@gmail.com>
License: GPL-2
VignetteBuilder: knitr
Imports: SummarizedExperiment, tximport, jsonlite, S4Vectors, IRanges,
        GenomicRanges, AnnotationDbi, GenomicFeatures, ensembldb,
        BiocFileCache, AnnotationHub, Biostrings, tibble, GenomeInfoDb,
        tools, utils, methods, Matrix
Suggests: knitr, rmarkdown, testthat, tximportData, org.Dm.eg.db,
        DESeq2, fishpond, edgeR, limma, devtools
URL: https://github.com/mikelove/tximeta
biocViews: Annotation, GenomeAnnotation, DataImport, Preprocessing,
        RNASeq, SingleCell, Transcriptomics, Transcription,
        GeneExpression, FunctionalGenomics, ReproducibleResearch,
        ReportWriting, ImmunoOncology
RoxygenNote: 7.2.1
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/tximeta
git_branch: RELEASE_3_16
git_last_commit: 189bd25
git_last_commit_date: 2022-11-01
Date/Publication: 2022-11-01
NeedsCompilation: no
Packaged: 2022-11-01 23:17:14 UTC; biocbuild
Author: Michael Love [aut, cre],
  Charlotte Soneson [aut, ctb],
  Peter Hickey [aut, ctb],
  Rob Patro [aut, ctb],
  NIH NHGRI [fnd],
  CZI [fnd]
