Package: tximeta
Version: 1.31.4
Title: Transcript Quantification Import with Automatic Metadata
Description: Transcript quantification import from Salmon and
    other quantifiers with automatic attachment of transcript ranges
    and release information, and other associated metadata. De novo
    transcriptomes can be linked to the appropriate sources with
    linkedTxomes and shared for computational reproducibility.
Authors@R: c(
    person("Michael", "Love", email="michaelisaiahlove@gmail.com", role = c("aut","cre")),
    person("Charlotte", "Soneson", role = c("aut","ctb")),
    person("Peter", "Hickey", role = c("aut","ctb")),
    person("Rob", "Patro", role = c("aut","ctb")),
    person("NIH NHGRI", role="fnd"),
    person("CZI", role="fnd")
    )
Maintainer: Michael Love <michaelisaiahlove@gmail.com>
License: GPL-2
VignetteBuilder: knitr, rmarkdown
Depends: R (>= 4.1.0)
Imports: SummarizedExperiment (>= 1.39.1), tximport, jsonlite,
        S4Vectors, IRanges, GenomicRanges (>= 1.61.1), AnnotationDbi,
        DBI, GenomicFeatures, txdbmaker, ensembldb, BiocFileCache,
        AnnotationHub, Biostrings, tibble, Seqinfo, tools, utils,
        methods, Matrix
Suggests: knitr, rmarkdown, testthat, tximportData (>= 1.37.5),
        org.Dm.eg.db, DESeq2, edgeR (>= 4.9.2), limma, devtools,
        macrophage
URL: https://thelovelab.github.io/tximeta,
        https://github.com/thelovelab/tximeta
biocViews: Annotation, GenomeAnnotation, DataImport, Preprocessing,
        RNASeq, LongRead, SingleCell, Transcriptomics, Transcription,
        GeneExpression, FunctionalGenomics, ReproducibleResearch,
        ReportWriting, ImmunoOncology
Roxygen: list(markdown = TRUE)
Encoding: UTF-8
Config/Needs/website: r-lib/pkgdown
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/tximeta
git_branch: devel
git_last_commit: 6825c7f
git_last_commit_date: 2026-06-15
Repository: Bioconductor 3.24
Date/Publication: 2026-06-16
NeedsCompilation: no
Packaged: 2026-06-17 00:43:34 UTC; biocbuild
Author: Michael Love [aut, cre],
  Charlotte Soneson [aut, ctb],
  Peter Hickey [aut, ctb],
  Rob Patro [aut, ctb],
  NIH NHGRI [fnd],
  CZI [fnd]
