% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizeToGene.R
\docType{methods}
\name{summarizeToGene,SummarizedExperiment-method}
\alias{summarizeToGene,SummarizedExperiment-method}
\title{Summarize estimated quantitites to gene-level}
\usage{
\S4method{summarizeToGene}{SummarizedExperiment}(object, varReduce = FALSE, ...)
}
\arguments{
\item{object}{a SummarizedExperiment produced by \code{tximeta}}

\item{varReduce}{whether to reduce per-sample inferential replicates
information into a matrix of sample variances \code{variance} (default FALSE)}

\item{...}{arguments passed to \code{tximport}}
}
\value{
a SummarizedExperiment with summarized quantifications
and transcript IDs as a CharacterList in the \code{mcols}
}
\description{
Summarizes abundances, counts, lengths, (and inferential
replicates or variance) from transcript- to gene-level.
Transcript IDs are stored as a CharacterList in the \code{mcols}
of the output object.
This function operates on SummarizedExperiment objects, and
will automatically access the relevant TxDb (by either finding it
in the BiocFileCache or by building it from an ftp location).
This function uses the tximport package to perform summarization,
where a method is defined that works on simple lists.
}
\examples{

example(tximeta)
gse <- summarizeToGene(se)

}
