% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dgelist.R
\name{makeDGEList}
\alias{makeDGEList}
\title{Make a DGEList from tximeta output}
\usage{
makeDGEList(se)
}
\arguments{
\item{se}{a SummarizedExperiment produced by tximeta}
}
\value{
a DGEList
}
\description{
A simple wrapper function for constructing a DGEList for use with edgeR.
See vignette for an example. Requires installation of the edgeR
package from Bioconductor.
}
