% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrievers.R
\name{retrieveDb}
\alias{retrieveDb}
\title{Retrieve the TxDb or EnsDb associated with a SummarizedExperiment}
\usage{
retrieveDb(se)
}
\arguments{
\item{se}{the SummarizedExperiment}
}
\value{
a database object
}
\description{
SummarizedExperiment objects returned by \code{\link{tximeta}} have
associated TxDb or EnsDb databases which are cached locally and used
to perform various metadata related tasks. This helper function
retrieves the database itself for the user to perform any additional
operations.
}
\examples{

example(tximeta)
edb <- retrieveDb(se)

}
